# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,dependency_constraint)
S3method(as.data.frame,depgraph_validation_report)
S3method(as.data.frame,graph_edge_set)
S3method(as.data.frame,graph_node_set)
S3method(as.data.frame,graph_query_result)
S3method(as.data.frame,leakage_risk_summary)
S3method(as.data.frame,split_constraint)
S3method(as.data.frame,split_spec)
S3method(as.data.frame,split_spec_validation)
S3method(plot,dependency_graph)
S3method(print,dependency_constraint)
S3method(print,dependency_graph)
S3method(print,depgraph_validation_report)
S3method(print,graph_edge_set)
S3method(print,graph_node_set)
S3method(print,graph_query_result)
S3method(print,leakage_risk_summary)
S3method(print,split_constraint)
S3method(print,split_spec)
S3method(print,split_spec_validation)
S3method(print,splitgraph_json_report)
S3method(summary,dependency_constraint)
S3method(summary,dependency_graph)
S3method(summary,depgraph_validation_report)
S3method(summary,graph_edge_set)
S3method(summary,graph_node_set)
S3method(summary,graph_query_result)
S3method(summary,leakage_risk_summary)
S3method(summary,split_constraint)
S3method(summary,split_spec)
S3method(summary,split_spec_validation)
export(as_igraph)
export(as_split_spec)
export(build_dependency_graph)
export(build_depgraph)
export(create_edges)
export(create_nodes)
export(dependency_constraint)
export(dependency_graph)
export(depgraph_validation_report)
export(derive_split_constraints)
export(detect_dependency_components)
export(detect_shared_dependencies)
export(graph_edge_set)
export(graph_from_metadata)
export(graph_node_set)
export(graph_query_result)
export(grouping_vector)
export(ingest_metadata)
export(leakage_risk_summary)
export(migrate_dependency_graph_json)
export(migrate_split_spec_json)
export(new_depgraph)
export(new_depgraph_edges)
export(new_depgraph_nodes)
export(query_edge_type)
export(query_neighbors)
export(query_node_type)
export(query_paths)
export(query_shortest_paths)
export(read_dependency_graph)
export(read_split_spec)
export(relatedness_edges_from_kinship)
export(spatial_edges_from_coords)
export(split_constraint)
export(split_spec)
export(split_spec_validation)
export(summarize_leakage_risks)
export(validate_depgraph)
export(validate_graph)
export(validate_graph_json)
export(validate_split_spec)
export(validate_split_spec_json)
export(write_dependency_graph)
export(write_split_spec)
importFrom(graphics,plot)
importFrom(igraph,ecount)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,vcount)
